SystemOrganization addCategory: #'Pier-FilesystemPersistence-Core'!
SystemOrganization addCategory: #'Pier-FilesystemPersistence-Tests'!

!PUMember class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 10/22/2010 12:48'!
buildFilesystemXmlDescription
	"Build a customized description for instances of the receiver that can be used
	for xml serialization."

	| children |
	
	children := self buildFilesystemXmlDescriptionChildren.
	children do: [ :each | each xmlReader: PRFilesystemXmlReader; xmlWriter: PRFilesystemXmlWriter ].

	^self description copyEmpty
		kind: self;
		xmlReader: PRFilesystemXmlReader;
		xmlWriter: PRFilesystemXmlWriter;
		setChildren: children;
		yourself! !

!PUMember class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 10/22/2010 11:03'!
buildFilesystemXmlDescriptionChildren

	^OrderedCollection new
		add: (self descriptionName
			accessor: (MASelectorAccessor read: #name write: #setName: );
			beWriteable;
			beXmlAttribute;
			yourself );
		add: (self descriptionComment
			beXmlElementCData;
			yourself );
		yourself
! !

!PUMember class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/19/2010 13:41'!
filesystemXmlDescription
	"Answer a description that can be used to serialize instances
	of the receiver to XML."

	^PRFilesystemXmlDescriptionBuilder for: self! !

!PUMember methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/19/2010 13:44'!
filesystemXmlDescription
	"Answer a customized description of the reciever that can be used
	to convert instances to xml and visa versa.	"

	^self class filesystemXmlDescription ! !

TestCase subclass: #PRFilesystemAccessTest
	instanceVariableNames: 'kernel newEmptyKernel context directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Tests'!

!PRFilesystemAccessTest methodsFor: 'accessing' stamp: 'jvds 8/17/2010 13:38'!
context
	^context! !

!PRFilesystemAccessTest methodsFor: 'accessing' stamp: 'jvds 8/17/2010 13:38'!
context: aPRContext
	context := aPRContext! !

!PRFilesystemAccessTest methodsFor: 'private' stamp: 'jvds 8/8/2010 14:47'!
exportToTempDir
	"self debug: #exportToTempDir"

	| access structure tempDir |

	tempDir := FSDiskFilesystem / 'tmp' / 'pier'.
	
	access := PRFilesystemAccess new 
			setDirectory: tempDir.
			
	structure := kernel root.
	self assert: structure environment notNil.
			
	access storeStructureTree: structure.
! !

!PRFilesystemAccessTest methodsFor: 'running' stamp: 'jvds 8/17/2010 14:20'!
setUp

	kernel := PRKernel named: 'fsaccess-test-kernel'.
	newEmptyKernel := PRKernel named: 'fsaccess-empty-kernel' root: ((PRPage named: 'empty')
		contents: 'nothing';
		yourself).
		
	context := PRContext kernel: kernel.
	directory := FSMemoryFilesystem new working! !

!PRFilesystemAccessTest methodsFor: 'running' stamp: 'jvds 8/17/2010 14:21'!
tearDown

	kernel notNil
		ifTrue: [ PRKernel instances remove: kernel ].
	newEmptyKernel notNil
		ifTrue: [ PRKernel instances remove: newEmptyKernel ]! !

!PRFilesystemAccessTest methodsFor: 'testing' stamp: 'jvds 8/17/2010 14:22'!
testFullStoreAndRead

	| access structure created path |
	
	access := PRFilesystemAccess new 
			setDirectory: directory.
			
	structure := kernel root.
			
	access storeStructureTree: structure.

	self context: (PRContext kernel: newEmptyKernel).
	PRCurrentContext 
		use: self
		during: [ created := access readRootStructure ].
	
	self assert: newEmptyKernel root = created.
	self assert: structure name = created name.
	self assert: created hasChildren.
	
	path := 'information/syntax'.
	self assert: (created lookupPath: path) name = 'syntax'.
	self assert: (structure lookupPath: path) title = (created lookupPath: path) title .! !

!PRFilesystemAccessTest methodsFor: 'testing' stamp: 'jvds 8/17/2010 14:23'!
testSimpleStoreAndRead

	| access structure created |
	
	access := PRFilesystemAccess new 
			setDirectory: directory.
			
	structure := kernel root.
			
	access storeStructure: structure.

	self context: (PRContext kernel: newEmptyKernel).
	PRCurrentContext 
		use: self
		during: [ created := access readRootStructure ].
	
	self assert: structure name = created name.
	self assert: structure title = created title.
	self assert: structure contents = created contents! !

!PRFilesystemAccessTest methodsFor: 'testing' stamp: 'jvds 8/7/2010 16:09'!
testStoreEnvironment

	| access root |
	
	access := PRFilesystemAccess new
			setDirectory: directory.
	
	root := kernel root.
	self assert: root environment notNil.

	access storeStructureTree: root.
	
	self assert: (access contentsDirectory / root name / 'environment') isDirectory! !

!PRFilesystemAccessTest methodsFor: 'testing' stamp: 'jvds 8/17/2010 14:23'!
testStoreEnvironmentAndRead

	| access structure created path |
	
	access := PRFilesystemAccess new 
			setDirectory: directory.
			
	structure := kernel root.
	self assert: structure environment notNil.
			
	access storeStructureTree: structure.

	self context: (PRContext kernel: newEmptyKernel).
	PRCurrentContext
		use: self
		during: [ created := access readRootStructure ].
	
	self assert: newEmptyKernel root == created.
	self assert: structure name = created name.
	self assert: created hasChildren.
	self assert: (created environment isKindOf: PRPage).
		
	path := 'information/syntax'.
	self assert: (created lookupPath: path) name = 'syntax'.
	self assert: (structure lookupPath: path) title = (created lookupPath: path) title.
! !

!PRFilesystemAccessTest methodsFor: 'testing' stamp: 'jvds 8/7/2010 14:27'!
testStoreTree

	| access root |
	
	access := PRFilesystemAccess new
			setDirectory: directory.
	
	root := kernel root.		
	access storeStructureTree: root.
	
	self assert: (access contentsDirectory / (root name, '.xml')) isFile.
	self assert: (access contentsDirectory / root name) isDirectory.
	self assert: (access contentsDirectory / root name) children notEmpty! !

TestCase subclass: #PRFilesystemPersistencyTest
	instanceVariableNames: 'kernel context filesystemAccess directory codec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Tests'!

!PRFilesystemPersistencyTest methodsFor: 'accessing' stamp: 'jvds 9/27/2010 22:37'!
context

	^context! !

!PRFilesystemPersistencyTest methodsFor: 'accessing' stamp: 'jvds 9/27/2010 22:40'!
context: aContext

	context := aContext! !

!PRFilesystemPersistencyTest methodsFor: 'accessing' stamp: 'jvds 9/27/2010 22:36'!
rootPageDirectory

	^filesystemAccess contentsDirectory / kernel root name ! !

!PRFilesystemPersistencyTest methodsFor: 'running' stamp: 'jvds 9/30/2010 18:36'!
setUp

	kernel := PRKernel named: 'fspersistency' root: ((PRPage named: 'empty')
		contents: 'nothing';
		yourself).
		
	kernel root addChild: ((PRPage named: 'page1')
		contents: '!!Page1';
		yourself).
		
	context := PRContext kernel: kernel.
		
	directory := FSMemoryFilesystem new working.
	filesystemAccess := PRFilesystemAccess new 
		setDirectory: directory.
		
	kernel persistency: (PRFilesystemPersistency new
		filesystemAccess: filesystemAccess).
		
	codec := GRCodec 	forEncoding: 'utf-8'.
! !

!PRFilesystemPersistencyTest methodsFor: 'running' stamp: 'jvds 9/27/2010 22:20'!
tearDown

	kernel notNil
		ifTrue: [ PRKernel instances remove: kernel ].! !

!PRFilesystemPersistencyTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 12:56'!
testAddCommand

	| command pageFile xml |

	command := PRAddCommand new
		name: 'newpage';
		type: PRPage.

	self context: (self context command: command).
	command execute.

	pageFile := self rootPageDirectory / 'newpage.xml'.
	self assert: pageFile isFile.

	xml := pageFile readStreamDo: [ :stream | XMLDOMParser parseDocumentFrom: stream contents asString readStream ].
	self assert: (xml root name = 'PRPage').
	self assert: (xml root at: 'name') = 'newpage'! !

!PRFilesystemPersistencyTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 12:57'!
testEditCommand

	| page document command pageFile xml |
	
	page := kernel root lookupPath: '/page1'.
	
	document := PRPage descriptionDocument fromString: '!!NewContents'.
	command := page editCommandClass new
		write: document using: PRPage descriptionDocument.
		
	self context: (self context structure: page command: command).
	command execute.

	pageFile := self rootPageDirectory / 'page1.xml'.
	self assert: pageFile isFile.

	xml := pageFile readStreamDo: [ :stream | XMLDOMParser parseDocumentFrom: stream contents asString readStream ].
	self assert: (xml root name = 'PRPage').
	self assert: (xml root at: 'name') = 'page1'.
	self assert: (xml root contentStringAt: 'contents') = '!!NewContents'..! !

!PRFilesystemPersistencyTest methodsFor: 'testing' stamp: 'jvds 9/30/2010 20:27'!
testMoveCommand

	| page command oldPageFile newPageFile xml |
	
	page := kernel root lookupPath: '/page1'.
	
	command := PRMoveCommand new
		name: 'page2'.
		
	self context: (self context structure: page command: command).
	command execute.

	oldPageFile := self rootPageDirectory / 'page1.xml'.
	self deny: oldPageFile exists.

	newPageFile := self rootPageDirectory / 'page2.xml'.
	self assert: newPageFile isFile.
	
	xml := newPageFile readStreamDo: [ :stream | XMLDOMParser parseDocumentFrom: stream contents asString readStream ].
	self assert: (xml root at: 'name') = 'page2'	
! !

!PRFilesystemPersistencyTest methodsFor: 'testing' stamp: 'jvds 9/28/2010 16:41'!
testRemoveCommand

	| page command pageFile |
	
	page := kernel root lookupPath: '/page1'.
	
	command := PRRemoveCommand new.
		
	self context: (self context structure: page command: command).
	command execute.

	pageFile := self rootPageDirectory / 'page1.xml'.
	self deny: pageFile exists.

! !

TestCase subclass: #PRXMLDescriptionsTest
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Tests'!

!PRXMLDescriptionsTest methodsFor: 'accessing' stamp: 'jvds 9/15/2010 14:28'!
context

	^context ifNil: [ context := PRContext kernel: (
		PRKernel named: 'test-kernel' root: (PRPage named: 'empty'))]
! !

!PRXMLDescriptionsTest methodsFor: 'accessing' stamp: 'jvds 9/15/2010 14:33'!
kernel

	^self context kernel! !

!PRXMLDescriptionsTest methodsFor: 'running' stamp: 'jvds 9/15/2010 14:32'!
tearDown

	context notNil
		ifTrue: [ PRKernel instances remove: context kernel ]! !

!PRXMLDescriptionsTest methodsFor: 'testing' stamp: 'jvds 8/31/2010 11:55'!
testGroup

	| group description xml created |
	
	group := PUGroup new
		setName: 'members'.
		
	description := group filesystemXmlDescription.
	xml := description toXmlNode: group.
	created := description fromXmlNode: xml.
	
	self assert: group name = created name.
! !

!PRXMLDescriptionsTest methodsFor: 'testing' stamp: 'jvds 8/31/2010 11:54'!
testPage

	| page description xml created |
	
	page := PRPage new
		name: 'simple-test-page';
		title: 'Simple Test Page';
		tags: #( 'world' 'simple');
		contents: '+hello world+'.
		
	description := page class filesystemXmlDescription.
	xml := description toXmlNode: page.
	created := description fromXmlNode: xml.
	
	self assert: page name = created name.
	self assert: page title = created title.
	self assert: page contents = created contents.
	self assert: page tags = created tags.! !

!PRXMLDescriptionsTest methodsFor: 'testing' stamp: 'jvds 8/31/2010 11:54'!
testUser

	| user description xml created |
	
	user := PUUser new
		setName: 'mrx';
		comment: 'just a test user';
		firstName: 'Pete';
		lastName: 'Bla';
		password: 'secret'.
		
	description := user filesystemXmlDescription.
	
	xml := description toXmlNode: user.	
	created := description fromXmlNode: xml.
	
	self assert: user name = created name.
	self assert: user comment = created comment.
	self assert: (created validatePassword: 'secret')
! !

!PRXMLDescriptionsTest methodsFor: 'testing' stamp: 'jvds 9/15/2010 14:38'!
testUserWithGroups

	| user group1 group2 description xml created |
	
	user := (PUUser named: 'mrx')
		comment: 'just a test user';
		firstName: 'Pete';
		lastName: 'Bla';
		password: 'secret'.
	self kernel users add: user.
		
	group1 := PUGroup named: 'admins'.
	group2 := PUGroup named: 'editors'.
	self kernel groups
		add: group1;
		add: group2.
	
	user
		addGroup: group1;
		addGroup: group2.
		
	description := user filesystemXmlDescription.
	xml := description toXmlNode: user.
	
	PRCurrentContext use: self during: [
		created := description fromXmlNode: xml ].
	
	self assert: user name = created name.
	self assert: user groups size = 2.
	self assert: user groups first name = group1 name.
	self assert: user groups last name = group2 name
! !

MADescriptionBuilder subclass: #PRFilesystemXmlDescriptionBuilder
	instanceVariableNames: ''
	classVariableNames: 'FilesystemDefault'
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemXmlDescriptionBuilder class methodsFor: 'accessing' stamp: 'jvds 8/7/2010 13:53'!
filesystemDefault
	^FilesystemDefault ifNil: [ FilesystemDefault := self new ]! !

!PRFilesystemXmlDescriptionBuilder class methodsFor: 'accessing' stamp: 'jvds 8/7/2010 13:53'!
filesystemDefault: aBuilder
	FilesystemDefault := aBuilder ! !

!PRFilesystemXmlDescriptionBuilder class methodsFor: 'building' stamp: 'jvds 8/7/2010 13:54'!
for: anObject
	^ self filesystemDefault for: anObject! !

!PRFilesystemXmlDescriptionBuilder class methodsFor: 'initialization' stamp: 'jvds 8/7/2010 13:55'!
unload
	self filesystemDefault finalize.
	self filesystemDefault: nil! !

!PRFilesystemXmlDescriptionBuilder methodsFor: 'private' stamp: 'jvds 8/7/2010 13:49'!
build: aClass

	^aClass buildFilesystemXmlDescription! !

MXXmlReader subclass: #PRFilesystemXmlReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemXmlReader methodsFor: 'private' stamp: 'jvds 8/7/2010 12:09'!
descriptionFor: aContainerObject

	^aContainerObject filesystemXmlDescription ! !

!PRFilesystemXmlReader methodsFor: 'visiting' stamp: 'jvds 8/8/2010 13:29'!
visitContainer: aDescription

	[ super visitContainer: aDescription ]
		on: PRFilesystemPostActionNotification do: [ :notification |
			(notification actionObject isNil and: [ self object notNil ])
				ifTrue: [ 
					notification 
						actionObject: self object;
						pass ]]! !

!PRFilesystemXmlReader methodsFor: 'visiting' stamp: 'jvds 10/22/2010 12:07'!
visitFileDescription: aDescription

	| fileNode fileModel |
	
	fileNode := aDescription xmlTarget getNodeFrom: self node.
	fileNode notNil
		ifTrue: [
			fileModel := aDescription kind new.
			fileModel
				filename: (fileNode attributeAt: 'filename');
				mimetype: (fileNode attributeAt: 'mimetype').
			fileModel isText
				ifTrue: [ fileModel contents: fileNode contentString ]
				ifFalse: [ fileModel contents: fileNode contentString base64Decoded ]].
	self object: fileModel ! !

!PRFilesystemXmlReader methodsFor: 'visiting' stamp: 'jvds 10/22/2010 11:43'!
visitStructureDescription: aDescription
	"It may not be possible to resolve the structure at this time. Use a notification to
	tell one of our callers that this should be done at a later time."
	
	| text |
	
	text := self readText: aDescription.
	text notNil
		ifTrue: [ PRFilesystemPostActionNotification write: text using: aDescription ]! !

!PRAddCommand methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/14/2010 14:45'!
doExecuteAndPersistWith: aFSPersistency
	"Execute the receiver and notify aFSPersistency about the changes."

	self doExecute.
	aFSPersistency handleNewStructure: answer structure! !

!PREditCommand methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/14/2010 14:45'!
doExecuteAndPersistWith: aFSPersistency
	"Execute the receiver and notify aFSPersistency about the changes."

	self doExecute.
	aFSPersistency handleUpdatedStructure: self structure! !

!PRCommand methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/14/2010 14:44'!
doExecuteAndPersistWith: aFSPersistency
	"Execute the receiver and notify aFSPersistency about the changes."
	
	self doExecute.
! !

PRExporterImporter subclass: #PRFilesystemExporterImporter
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemExporterImporter class methodsFor: 'testing' stamp: 'jvds 10/22/2010 13:38'!
isAbstract
	^false! !

!PRFilesystemExporterImporter class methodsFor: 'accessing' stamp: 'jvds 10/22/2010 13:38'!
label
	^ 'Filesystem XML'! !

!PRFilesystemExporterImporter methodsFor: 'actions' stamp: 'jvds 10/22/2010 14:37'!
export: aKernel on: aRequestContext
	"Export all structures as a zipped directory tree with xml files."
	
	| ref archive bytes |
	
	ref := FSDiskFilesystem imageDirectory / 'pier-export.zip'.
	archive := FSZipFilesystem atReference: ref.
	archive open.
	aKernel exportToFilesystem: archive root.
	archive close.
	
	ref readStreamDo: [ :stream | bytes := stream contents ].
	ref delete.
	
	aRequestContext response
		document: bytes
			mimeType: self mimeType
			fileName: aKernel name , self extension;
		respond! !

!PRFilesystemExporterImporter methodsFor: 'accessing' stamp: 'jvds 10/22/2010 14:11'!
extension
	^'.zip'! !

!PRFilesystemExporterImporter methodsFor: 'accessing' stamp: 'jvds 10/22/2010 14:12'!
mimeType
	^ 'application/zip'! !

!PRCase class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 10/22/2010 11:05'!
buildFilesystemXmlDescriptionChildren

	| children accessorsToSkip attribAccessors child |
	
	children := super buildFilesystemXmlDescriptionChildren.
	
	accessorsToSkip := OrderedCollection new
		add: self descriptionDocument accessor;
		add: #plainText asAccessor;
		yourself.

	children := children reject: [ :each | accessorsToSkip includes: each accessor ].
	
	children add: (MAMemoDescription new
		accessor: #contents;
		beXmlElementCData ).
	
	^children! !

!PRRemoveCommand methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/14/2010 14:45'!
doExecuteAndPersistWith: aFSPersistency
	"Execute the receiver and notify aFSPersistency about the changes."
	
	| path |
	
	path := self structure filesystemPath.
	self doExecute.
	aFSPersistency handleRemovedStructureWithPath: path! !

!PUUser class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 10/22/2010 11:04'!
buildFilesystemXmlDescriptionChildren
	"password token groups superuser firstName lastName email"
	
	^super buildFilesystemXmlDescriptionChildren
		add: (MANumberDescription new
			accessor: #passwordHash;
			beWriteable;
			beXmlElement;
			yourself );
		add: (self descriptionSuperuser
			beWriteable;
			beXmlElement;
			yourself );
		add: (self descriptionFirstName
			beWriteable;
			beXmlElement;
			yourself );
		add: (self descriptionLastName
			beWriteable;
			beXmlElement;
			yourself );
		add: (self descriptionEmail
			beWriteable;
			beXmlElement;
			yourself );
		add: (MAMultipleOptionDescription new
			accessor: #groupNames;
			beXmlElement;
			yourself );
		yourself
! !

!PUUser methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/31/2010 22:48'!
groupNames

	^self groups collect: [ :each | each name ]! !

!PUUser methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 9/15/2010 14:41'!
groupNames:  aStringColl

	| kernel group |
	
	(aStringColl isNil or: [ aStringColl isEmpty ])
		ifTrue: [ ^self ].
	kernel := PRCurrentContext value kernel.
	aStringColl do: [ :each | 
		group := kernel groupNamed: each ifNone: [ nil ].
		group notNil
			ifTrue: [ self addGroup: group ]]! !

!PUUser methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/19/2010 14:10'!
passwordHash
	^password! !

!PUUser methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/19/2010 14:13'!
passwordHash: anObject
	password := anObject! !

!PRMoveCommand methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/14/2010 14:45'!
doExecuteAndPersistWith: aFSPersistency
	"Execute the receiver and notify aFSPersistency about the changes."

	| oldPath |

	oldPath := self structure filesystemPath.
	self doExecute.
	aFSPersistency handleMovedStructure: self structure from: oldPath! !

PRHistoryPersistency subclass: #PRFilesystemPersistency
	instanceVariableNames: 'filesystemAccess directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemPersistency methodsFor: 'accessing' stamp: 'jvds 8/10/2010 17:18'!
directory

	^directory! !

!PRFilesystemPersistency methodsFor: 'accessing' stamp: 'jvds 8/10/2010 17:18'!
directory: aString

	self filesystemAccess: (PRFilesystemAccess new
		setDirectory: (FSDiskFilesystem current resolveString: aString);
		yourself).
	directory := aString! !

!PRFilesystemPersistency methodsFor: 'actions' stamp: 'jvds 8/10/2010 13:17'!
execute: aCommand

	"no actions required for read-only commands"
	aCommand isLogged
		ifFalse: [ ^self apply: aCommand ].
		
	self mutex critical: [
		aCommand isChecked
			ifTrue: [ aCommand doValidate ].
		aCommand doExecuteAndPersistWith: self.
		self add: aCommand context ]! !

!PRFilesystemPersistency methodsFor: 'accessing' stamp: 'jvds 8/2/2010 22:13'!
filesystemAccess
	^filesystemAccess ! !

!PRFilesystemPersistency methodsFor: 'accessing' stamp: 'jvds 8/10/2010 17:20'!
filesystemAccess: aFilesystemAccess
	filesystemAccess := aFilesystemAccess.
	directory := filesystemAccess directory printString ! !

!PRFilesystemPersistency methodsFor: 'command callbacks' stamp: 'jvds 9/29/2010 21:43'!
handleMovedStructure: aPRPage from: aFSPath 
	self filesystemAccess
		storeStructure: aPRPage;
		removeStructureTreeWithPath: aFSPath! !

!PRFilesystemPersistency methodsFor: 'command callbacks' stamp: 'jvds 8/10/2010 13:38'!
handleNewStructure: aStructure

	self filesystemAccess storeStructureTree: aStructure ! !

!PRFilesystemPersistency methodsFor: 'command callbacks' stamp: 'jvds 8/10/2010 13:43'!
handleRemovedStructureWithPath: aString

	self filesystemAccess removeStructureTreeWithPath: aString! !

!PRFilesystemPersistency methodsFor: 'command callbacks' stamp: 'jvds 8/10/2010 13:40'!
handleUpdatedStructure: aStructure

	self filesystemAccess storeStructure: aStructure ! !

!PRFilesystemPersistency methodsFor: 'initialization' stamp: 'jvds 9/27/2010 22:47'!
setKernel: aKernel

	super setKernel: aKernel.
	(aKernel notNil and: [ self filesystemAccess notNil ])
		ifTrue: [ self filesystemAccess storeStructureTree: aKernel root ]! !

!PRKernel class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/14/2010 16:10'!
importFromDisk: aPath named: aString
	"Create a new kernel with the name aString and import the
	contents from disk."
	
	| filesystem |
	
	filesystem := FSDiskFilesystem current.
	self 
		importFromFilesystem: (FSReference filesystem: filesystem path: (aPath asResolvedBy: filesystem))
		named: aString! !

!PRKernel class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/18/2010 23:26'!
importFromFilesystem: aFSReference named: aString
	"Create a new kernel with the name aString and import the
	contents from aFSReference."
	
	| access kernel holder |
	
	access := PRFilesystemAccess new 
			setDirectory: aFSReference;
			yourself.
			
	kernel := self named: aString root: (PRPage named: 'dummy').
	holder := PRFilesystemContextHolder new
			context: (PRContext kernel: kernel);
			yourself.
	PRCurrentContext 
		use: holder
		during: [ access readRootStructure ].
	^kernel
! !

!PRKernel methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/14/2010 15:33'!
exportToDisk: aPath
	"Export all information about this Wiki to disk."

	| filesystem |
	
	filesystem := FSDiskFilesystem current.
	self exportToFilesystem: (FSReference filesystem: filesystem path: (aPath asResolvedBy: filesystem))! !

!PRKernel methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 10/22/2010 13:53'!
exportToFilesystem: aFSReference

	| access |

	access := PRFilesystemAccess new 
			setDirectory: aFSReference;
			yourself.
			
	access storeStructureTree: self root.
	self users do: [ :user | access storeUser: user ].
	self groups do: [ :group | access storeGroup: group ]! !

Object subclass: #PRFilesystemAccess
	instanceVariableNames: 'contentsDirectory usersDirectory groupsDirectory codec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemAccess methodsFor: 'accessing' stamp: 'jvds 8/2/2010 20:58'!
contentsDirectory
	^contentsDirectory ! !

!PRFilesystemAccess methodsFor: 'accessing' stamp: 'jvds 8/10/2010 17:19'!
directory

	^contentsDirectory parent! !

!PRFilesystemAccess methodsFor: 'accessing' stamp: 'jvds 8/7/2010 15:01'!
directoryForChildrenOf: aFSReference

	| name |
	
	"chop off the '.xml' extension"
	name := aFSReference basename allButLast: 4.
	^aFSReference parent / name! !

!PRFilesystemAccess methodsFor: 'accessing' stamp: 'jvds 8/2/2010 21:57'!
groupsDirectory
	^groupsDirectory ! !

!PRFilesystemAccess methodsFor: 'initialize-release' stamp: 'jvds 9/30/2010 18:26'!
initialize

	super initialize.
	codec := GRCodec forEncoding: 'utf-8'.! !

!PRFilesystemAccess methodsFor: 'accessing' stamp: 'jvds 8/17/2010 13:51'!
kernel

	^PRCurrentContext value ifNotNil: [ :context | context kernel ]! !

!PRFilesystemAccess methodsFor: 'public' stamp: 'jvds 9/29/2010 21:18'!
moveStructureTreeFrom: aFSPath to: aNewFSPath

	| fileRef dirRef |
	
	fileRef := self contentsDirectory resolvePath: (aFSPath withExtension: 'xml').
	fileRef isFile
		ifTrue: [ fileRef delete ].
	
! !

!PRFilesystemAccess methodsFor: 'public' stamp: 'jvds 8/18/2010 23:18'!
readRootStructure

	| rootReference |
	
	rootReference := self contentsDirectory children 
		detect: [ :ref | ref isFile and: [ ref basename endsWith: '.xml' ]]
		ifNone: [ self error: 'No contents found in ', self contentsDirectory printString ].
		
	^self readStructureTree: rootReference! !

!PRFilesystemAccess methodsFor: 'reading' stamp: 'jvds 9/30/2010 18:33'!
readStructureAt: aFSReference

	aFSReference readStreamDo: [ :stream |
		^self readStructureWith: PRStructure filesystemXmlDescription from: (codec decode: stream contents) ]! !

!PRFilesystemAccess methodsFor: 'reading' stamp: 'jvds 8/17/2010 13:49'!
readStructureTree: aFSReference

	| postActions node |
	
	postActions := OrderedCollection new.
	[
		node := self readStructureTreeAt: aFSReference parent: nil
	] on: PRFilesystemPostActionNotification do: [ :notification |
		postActions add: notification action.
		notification resume ].

	PRCurrentContext value: (PRCurrentContext value structure: node).
	postActions do: [ :each | each perform ].
	
	^node! !

!PRFilesystemAccess methodsFor: 'reading' stamp: 'jvds 8/16/2010 21:52'!
readStructureTreeAt: aFSReference parent: parent

	| node childrenDir |
	
	node := self readStructureAt: aFSReference.
	parent isNil
		ifTrue: [ self kernel ifNotNil: [ :kernel | kernel root: node ]]
		ifFalse: [ parent addChild: node ].
	
	childrenDir := (self directoryForChildrenOf: aFSReference).
	childrenDir isDirectory 
		ifTrue: [
			childrenDir children do: [ :each |
				(each basename endsWith: '.xml')  
					ifTrue: [ self readStructureTreeAt: each parent: node ] ] ].
	^node! !

!PRFilesystemAccess methodsFor: 'reading' stamp: 'jvds 9/30/2010 18:27'!
readStructureWith: aDescription from: aStream

	^aDescription fromXml: aStream! !

!PRFilesystemAccess methodsFor: 'public' stamp: 'jvds 8/10/2010 16:40'!
removeStructureTreeWithPath: aFSPath

	| fileRef dirRef |
	
	fileRef := self contentsDirectory resolvePath: (aFSPath withExtension: 'xml').
	fileRef isFile
		ifTrue: [ fileRef delete ].
	
	dirRef := self contentsDirectory resolvePath: aFSPath.
	dirRef isDirectory
		ifTrue: [ dirRef delete ]! !

!PRFilesystemAccess methodsFor: 'accessing' stamp: 'jvds 8/5/2010 15:50'!
setDirectory: aDirectoryReference

	contentsDirectory := aDirectoryReference / 'contents'.
	contentsDirectory ensureDirectory.
	
	usersDirectory := aDirectoryReference / 'users'.
	usersDirectory ensureDirectory.
	
	groupsDirectory := aDirectoryReference / 'groups'.
	groupsDirectory ensureDirectory.! !

!PRFilesystemAccess methodsFor: 'storing' stamp: 'jvds 9/30/2010 21:50'!
storeGroup: aGroup

	self groupsDirectory / (aGroup name, '.xml') writeStreamDo: [ :stream |
		(codec encoderFor: stream) 
			nextPutAll: (aGroup filesystemXmlDescription toXml: aGroup)]! !

!PRFilesystemAccess methodsFor: 'public' stamp: 'jvds 8/10/2010 16:21'!
storeStructure: aStructure

	| ref |
	
	ref := self contentsDirectory resolvePath: (aStructure filesystemPath withExtension: 'xml').
	ref parent ensureDirectory.
	ref writeStreamDo: [ :stream |
		self storeStructureXml: aStructure on: stream ]! !

!PRFilesystemAccess methodsFor: 'public' stamp: 'jvds 8/2/2010 21:51'!
storeStructureTree: aStructure

	aStructure enumerator everything do: [ :each | self storeStructure: each ]! !

!PRFilesystemAccess methodsFor: 'storing' stamp: 'jvds 8/31/2010 12:07'!
storeStructureXml: aStructure on: aStream

	| description doc writer buffer |
	
	description := aStructure filesystemXmlDescription.
	doc := description toXmlNode: aStructure.
		
	buffer := String new writeStream.
	writer := (XMLWriter on: buffer)
		"prettyPrint;"
		yourself.
		
	doc printXMLOn: writer.
	
	aStream nextPutAll: buffer contents
! !

!PRFilesystemAccess methodsFor: 'storing' stamp: 'jvds 9/30/2010 21:47'!
storeUser: aUser

	self usersDirectory / (aUser name, '.xml') writeStreamDo: [ :stream |
		stream nextPutAll: (aUser filesystemXmlDescription toXml: aUser)]! !

!PRFilesystemAccess methodsFor: 'accessing' stamp: 'jvds 8/2/2010 21:57'!
usersDirectory
	^usersDirectory ! !

Object subclass: #PRFilesystemContextHolder
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemContextHolder methodsFor: 'accessing' stamp: 'jvds 8/18/2010 23:08'!
context
	^ context! !

!PRFilesystemContextHolder methodsFor: 'accessing' stamp: 'jvds 8/18/2010 23:08'!
context: anObject
	context := anObject! !

Object subclass: #PRFilesystemPostAction
	instanceVariableNames: 'object value description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemPostAction methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:03'!
description
	^ description! !

!PRFilesystemPostAction methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:03'!
description: anObject
	description := anObject! !

!PRFilesystemPostAction methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:03'!
object
	^ object! !

!PRFilesystemPostAction methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:03'!
object: anObject
	object := anObject! !

!PRFilesystemPostAction methodsFor: 'actions' stamp: 'jvds 8/16/2010 21:41'!
perform

	| struct |
	
	struct := self description fromString: self value.
	self object write: struct using: self description! !

!PRFilesystemPostAction methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:03'!
value
	^ value! !

!PRFilesystemPostAction methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:03'!
value: anObject
	value := anObject! !

MXXmlWriter subclass: #PRFilesystemXmlWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemXmlWriter methodsFor: 'visiting-description' stamp: 'jvds 10/22/2010 12:19'!
visitFileDescription: aDescription

	| fileModel fileNode |
	
	fileModel := self object readUsing: aDescription.
	fileModel notNil
		ifTrue: [
			fileNode := aDescription xmlTarget addNodeTo: self containerNode.
			fileNode
				attributeAt: 'filename' put: fileModel filename;
				attributeAt: 'mimetype' put: fileModel mimetype.
			fileModel isText
				ifTrue: [ fileNode addContent: fileModel contents asString ]
				ifFalse: [ fileNode addContent: fileModel contents base64Encoded ]].
	self node: fileNode.! !

!PRComponent methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 10/22/2010 12:49'!
filesystemXmlDescription
	"Answer a customized description of the reciever that can be used
	to convert instances to xml and visa versa.	"

	^super filesystemXmlDescription copy
		addAll: (self componentDescription collect: [ :descr |
			descr copy
				beXmlElement;
				xmlReader: PRFilesystemXmlReader;
				xmlWriter: PRFilesystemXmlWriter;
				yourself ]);
		yourself! !

GRNotification subclass: #PRFilesystemPostActionNotification
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FilesystemPersistence-Core'!

!PRFilesystemPostActionNotification class methodsFor: 'as yet unclassified' stamp: 'jvds 8/8/2010 13:06'!
update: anObject write: aValue using: aDescription

	^self new
		action: (PRFilesystemPostAction new
			object: anObject;
			value: aValue;
			description: aDescription);
		signal! !

!PRFilesystemPostActionNotification class methodsFor: 'as yet unclassified' stamp: 'jvds 8/8/2010 13:16'!
write: aValue using: aDescription

	^self update: nil write: aValue using: aDescription ! !

!PRFilesystemPostActionNotification methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:04'!
action
	^ action! !

!PRFilesystemPostActionNotification methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:04'!
action: anObject
	action := anObject! !

!PRFilesystemPostActionNotification methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:30'!
actionObject
	^self action object! !

!PRFilesystemPostActionNotification methodsFor: 'accessing' stamp: 'jvds 8/8/2010 13:29'!
actionObject: anObject
	self action object: anObject! !

!PRStructure class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 10/22/2010 12:50'!
buildFilesystemXmlDescription
	"Build a customized description for instances of the receiver that can be used
	for xml serialization."

	| children |
	
	children := self buildFilesystemXmlDescriptionChildren.
	children do: [ :each | each xmlReader: PRFilesystemXmlReader; xmlWriter: PRFilesystemXmlWriter ].

	^self description copyEmpty
		kind: self;
		xmlReader: PRFilesystemXmlReader;
		xmlWriter: PRFilesystemXmlWriter;
		setChildren: children;
		yourself! !

!PRStructure class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 10/22/2010 11:05'!
buildFilesystemXmlDescriptionChildren

	| children accessorsToSkip attribAccessors child |
	
	children := OrderedCollection new.
	
	accessorsToSkip := OrderedCollection new
		add: #owner asAccessor;
		add: #group asAccessor;
		yourself.
		
	attribAccessors := Array with: #name asAccessor.
	
	self description children do: [ :each |
		(accessorsToSkip includes: each accessor)
			ifFalse: [
				child := children add: each copy.
				(attribAccessors includes: child accessor)
					ifTrue: [	child beXmlAttribute ]
					ifFalse: [ child beXmlElement ] ] ].
					
	"only include menu tag if the property is set"
	(children detect: [ :each | each accessor = #menu asAccessor ] ifNone: [ nil ])
		ifNotNil: [ :description | 
			description
				accessor: (MAPluggableAccessor 
					read: [ :o | o propertyAt: #menu ifAbsent: [ nil ]] 
					write: [ :o :v | v notNil ifTrue: [ o menu: v ]])].

	"only include hideFromMenus tag if it has the value true"	
	(children detect: [ :each | each accessor = #hideFromMenus asAccessor ] ifNone: [ nil ])
		ifNotNil: [ :description | 
			description
				accessor: (MAPluggableAccessor 
					read: [ :o | o propertyAt: #hideFromMenus ifAbsent: [ nil ]] 
					write: [ :o :v | v notNil ifTrue: [ o hideFromMenus: v ]])].

	^children! !

!PRStructure class methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/7/2010 14:13'!
filesystemXmlDescription
	"Answer a description that can be used to serialize instances
	of the receiver to XML."

	^PRFilesystemXmlDescriptionBuilder for: self! !

!PRStructure methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/10/2010 14:25'!
filesystemPath
	"Answer a FSPath instance with an absolute path from the root to the receiver."

	| coll pnode |
	
	coll := OrderedCollection new.
	pnode := self.
	[ pnode notNil ] whileTrue: [ coll add: pnode name. pnode := pnode parent ].
	^FSPath withAll: coll reversed! !

!PRStructure methodsFor: '*pier-filesystempersistence-core' stamp: 'jvds 8/14/2010 14:47'!
filesystemXmlDescription
	"Answer a customized description of the reciever that can be used
	to convert instances to xml and visa versa.	"

	^self class filesystemXmlDescription ! !
